﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml.Linq;
using System.ComponentModel;

namespace ComponentGeek.Extender
{
    /// <summary>
    /// FreezeHeaderExtender 
    /// </summary>
    [TargetControlType(typeof(Control))]
    public class FreezeHeaderExtender : ExtenderControl
    {
        public FreezeHeaderExtender() { }

        /// <summary>   
        /// ヘッダテーブルのCssClass   
        /// </summary>   
        public string HeaderCssClass { get; set; }
        /// <summary>   
        /// ボディテーブルのCssClass   
        /// </summary>   
        public string BodyCssClass { get; set; }
        /// <summary>   
        /// ボディ部の幅   
        /// </summary>   
        public Unit Width { get; set; }
        /// <summary>   
        /// ボディ部の高さ   
        /// </summary>   
        public Unit Height { get; set; }
        private FreezeHeaderAdjustWidthMode _widthAdjustMode = FreezeHeaderAdjustWidthMode.Header;
        /// <summary>   
        /// 幅調整モード   
        /// </summary>   
        [DefaultValue(FreezeHeaderAdjustWidthMode.Header)]
        public FreezeHeaderAdjustWidthMode WidthAdjustMode
        {
            get { return _widthAdjustMode; }
            set { _widthAdjustMode = value; }
        }
        public int _headerColumnCount = 0;
        /// <summary>   
        /// ヘッダ列の数   
        /// </summary>   
        [DefaultValue(0)]
        public int HeaderColumnCount
        {
            get { return _headerColumnCount; }
            set { _headerColumnCount = value; }
        }
        public int _headerRowCount = 1;
        /// <summary>   
        /// ヘッダ行の行数   
        /// </summary>   
        [DefaultValue(1)]
        public int HeaderRowCount
        {
            get { return _headerRowCount; }
            set { _headerRowCount = value; }
        }

        protected override IEnumerable<ScriptDescriptor> GetScriptDescriptors(System.Web.UI.Control targetControl)
        {
            if (!targetControl.HasControls()) return new ScriptBehaviorDescriptor[] { };

            ScriptBehaviorDescriptor descriptor = new ScriptBehaviorDescriptor("ComponentGeek.Extender.FreezeHeaderBehavior", targetControl.ClientID);
            if (!string.IsNullOrEmpty(HeaderCssClass))
            {
                descriptor.AddProperty("headerCssClass", HeaderCssClass);
            }
            if (!string.IsNullOrEmpty(BodyCssClass))
            {
                descriptor.AddProperty("bodyCssClass", BodyCssClass);
            }
            if (Height != null)
            {
                descriptor.AddProperty("height", Height.ToString());
            }
            if (Width != null)
            {
                descriptor.AddProperty("width", Width.ToString());
            }
            descriptor.AddProperty("widthAdjustMode", WidthAdjustMode);
            descriptor.AddProperty("headerColumnCount", HeaderColumnCount);
            descriptor.AddProperty("headerRowCount", HeaderRowCount);

            return new ScriptBehaviorDescriptor[] { descriptor };
        }

        // スクリプト参照を生成します
        protected override IEnumerable<ScriptReference> GetScriptReferences()
        {
            yield return new ScriptReference("ComponentGeek.Extender.FreezeHeaderBehavior.js", this.GetType().Assembly.FullName);
            yield return new ScriptReference("ComponentGeek.Extender.TableResizeBehavior.js", this.GetType().Assembly.FullName);
        }
    }
    /// <summary>   
    /// 幅調整モード   
    /// - Default : 幅の広いほうに合わせる   
    /// - Header  : ヘッダ行の幅に合わせる   
    /// - Data    : データ行の幅に合わせる   
    /// </summary>   
    public enum FreezeHeaderAdjustWidthMode
    {
        Default = 0,
        Header,
        Data
    }
}